@@ -4,4 +4,4 @@ A productivity enhancer for creating Cypher queries in Python
 
 from .version import __version__
 from .builder import QueryBuilder
-from .typedefs import Mapping, Properties
+from .typedefs import Mapping
