@@ -3,7 +3,7 @@
 # pylint: disable=R0901
 # pylint: disable=R0903
 # pylint: disable=W0102
-from typing import List
+from typing import List, Union
 from .typedefs import Mapping, Properties
 
 
@@ -31,6 +31,10 @@ class Query():
         """Get the final query string ."""
         return str(self)
 
+    def cypher(self, cypher_query_str):
+        """Concatenate a cypher query string"""
+        return AnyAvailable(self.query.strip() + ' ' + cypher_query_str.strip())
+
 
 class QueryStart(Query):
     """A class for representing a "QUERY START" clause."""
@@ -51,11 +55,11 @@ class Call(Query):
 class CaseWhen(Query):
     """A class for representing a "CASE WHEN" clause."""
 
-    def case_when(self, filters: Properties, on_true: str, on_false: str, ref_name: str, comparison_operator: str = "=", boolean_operator: str = "AND"):
+    def case_when(self, filters: dict, on_true: str, on_false: str, ref_name: str, comparison_operator: str = "=", boolean_operator: str = "AND"):
         """Concatenate a CASE WHEN clause to the query, created from a list of given property filters.
 
-        :param filters: A Properties object that represents the set of properties to be filtered
-        :type filters: Properties
+        :param filters: A dict representing the set of properties to be filtered
+        :type filters: dict
         :param on_true: The query to run when the predicate is true
         :type on_true: str
         :param on_false: The query to run when the predicate is false
@@ -76,6 +80,18 @@ class CaseWhen(Query):
         return CaseWhenAvailable(self.query + filt)
 
 
+class Create(Query):
+    """A class for representing a "CREATE" clause."""
+
+    def create(self):
+        """Concatenate the "CREATE" clause.
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: CreateAvailable
+        """
+        return CreateAvailable(self.query + ' CREATE')
+
+
 class Delete(Query):
     """A class for representing a "DELETE" clause."""
 
@@ -89,7 +105,7 @@ class Delete(Query):
         :rtype: DeleteAvailable
         """
         ret = f' DELETE {ref_name}'
-        return Query(self.query + ret)
+        return DeleteAvailable(self.query + ret)
 
     def detach_delete(self, ref_name: str):
         """Concatenate a DETACH DELETE clause for a referenced instance from the DB.
@@ -101,44 +117,24 @@ class Delete(Query):
         :rtype: DeleteAvailable
         """
         ret = f' DETACH DELETE {ref_name}'
-        return Query(self.query + ret)
+        return DeleteAvailable(self.query + ret)
 
 
-class Where(Query):
-    """A class for representing a "WHERE" clause."""
+class Limit(Query):
+    """A class for representing a "LIMIT" clause."""
 
-    def where(self, name: str, comparison_operator: str, value: str):
-        """Concatenate a WHERE clause to the query, created as {name} {comparison_operator} {value}. E.g. x = 'abc'.
+    def limit(self, limitation: Union[int, str]):
+        """Concatenate a limit statement.
 
-        :param name: The name of the object which is to be used in the comparison
-        :type name: str
-        :param comparison_operator: A string operator, according to which the comparison between compared object and
-            the {value} is done, e.g. for "=", we get: {name} = {value}
-        :type comparison_operator: str
-        :param value: The value which is compared against
-        :type value: str
+        :param limitation: A non-negative integer or a string of cypher expression that evaluates to a non-negative
+            integer (as long as it is not referring to any external variables)
+        :type limitation: Union[int, str]
 
         :return: A Query object with a query that contains the new clause.
-        :rtype: WhereAvailable
+        :rtype: LimitAvailable
         """
-        return self.where_multiple({name: value}, comparison_operator)
-
-    def where_multiple(self, filters: Properties, comparison_operator: str = "=", boolean_operator: str = ' AND '):
-        """Concatenate a WHERE clause to the query, created from a list of given property filters.
-
-        :param filters: A Properties object that represents the set of properties to be filtered
-        :type filters: Properties
-        :param comparison_operator: A string operator, according to which the comparison between property values is
-            done, e.g. for "=", we get: property.name = property.value, defaults to "="
-        :type comparison_operator: str
-        :param boolean_operator: The boolean operator to apply between predicates, defaults to ' AND '
-        :type boolean_operator: str
-
-        :return: A Query object with a query that contains the new clause.
-        :rtype: WhereAvailable
-        """
-        filt = ' WHERE ' + Properties(filters).to_str(comparison_operator, boolean_operator)
-        return WhereAvailable(self.query + filt)
+        ret = f" LIMIT {limitation}"
+        return LimitAvailable(self.query + ret)
 
 
 class Match(Query):
@@ -176,16 +172,15 @@ class Merge(Query):
 class Node(Query):
     """A class for representing a "NODE" clause."""
 
-    def node(self, labels: List[str] = None, ref_name: str = None, properties: Properties = None):
+    def node(self, labels: List[str] = None, ref_name: str = None, properties: dict = None):
         """Concatenate a graph Node, which may be filtered using any label/s and/or property/properties.
 
         :param labels: The neo4j label (or list of labels) for that node, defaults to None
         :type labels: List[str]
         :param ref_name: A reference name to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param properties: A Properties object that represents the set of properties by which the nodes are filtered,
-            defaults to None
-        :type properties: Properties
+        :param properties: A dict representing the set of properties by which the nodes are filtered, defaults to None
+        :type properties: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: NodeAvailable
@@ -204,28 +199,30 @@ class Node(Query):
 
         ref_name = ref_name or ''
 
-        if not (self.query.endswith('-') or self.query.endswith('>') or self.query.endswith('<')):
-            self.query += ' '
+        query = self.query
+        if not (query.endswith('-') or query.endswith('>') or query.endswith('<')):
+            query += ' '
+
+        query += f'({ref_name}{labels_string}{property_string})'
 
         if isinstance(self, MergeAvailable):
-            return NodeAfterMergeAvailable(self.query + f'({ref_name}{labels_string}{property_string})')
+            return NodeAfterMergeAvailable(query)
 
-        return NodeAvailable(self.query + f'({ref_name}{labels_string}{property_string})')
+        return NodeAvailable(query)
 
 
 class NodeAfterMerge(Query):
     """A class for representing a "NODE AFTER MERGE" clause."""
 
-    def node(self, labels: List[str] = None, ref_name: str = None, properties: Properties = None):
+    def node(self, labels: List[str] = None, ref_name: str = None, properties: dict = None):
         """Concatenate a graph Node, which may be filtered using any label/s and/or property/properties.
 
         :param labels: The neo4j label (or list of labels) for that node, defaults to None
         :type labels: List[str]
         :param ref_name: A reference name to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param properties: A Properties object that represents the set of properties by which the nodes are filtered,
-            defaults to None
-        :type properties: Properties
+        :param properties: A dict representing the set of properties by which the nodes are filtered, defaults to None
+        :type properties: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: NodeAfterMergeAvailable
@@ -244,13 +241,16 @@ class NodeAfterMerge(Query):
 
         ref_name = ref_name or ''
 
-        if not (self.query.endswith('-') or self.query.endswith('>') or self.query.endswith('<')):
-            self.query += ' '
+        query = self.query
+        if not (query.endswith('-') or query.endswith('>') or query.endswith('<')):
+            query += ' '
+
+        query += f'({ref_name}{labels_string}{property_string})'
 
         if isinstance(self, MergeAvailable):
-            return NodeAfterMergeAvailable(self.query + f'({ref_name}{labels_string}{property_string})')
+            return NodeAfterMergeAvailable(query)
 
-        return NodeAvailable(self.query + f'({ref_name}{labels_string}{property_string})')
+        return NodeAvailable(query)
 
 
 class OnCreate(Query):
@@ -286,92 +286,120 @@ class OperatorEnd(Query):
         :return: A Query object with a query that contains the new clause.
         :rtype: OperatorEndAvailable
         """
-        return OperatorEndAvailable(self.query + ')')
+        return OperatorEndAvailable(self.query + ' )')
 
 
 class OperatorStart(Query):
     """A class for representing a "OPERATOR START" clause."""
 
-    def operator_start(self, operator: str, ref_name: str = None, args: Properties = None):
+    def operator_start(self, operator: str, ref_name: str = None, args: dict = None):
         """Concatenate an operator (e.g. ShortestPath), where its result may be given a name for future reference.
 
         :param operator: The neo4j operator to be used (e.g. ShortestPath)
         :type operator: str
         :param ref_name: A reference name of the result, to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param args: A string of arguments, to be passed to the operator function, defaults to None
-        :type args: Properties
+        :param args: A dict of arguments, to be passed to the operator function, defaults to None
+        :type args: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: OperatorStartAvailable
         """
         result_name = '' if ref_name is None else f'{ref_name} = '
-        arguments = '' if args is None else f'{args}'
+        arguments = '' if args is None else f' {args}'
 
         return OperatorStartAvailable(self.query + f' {result_name}{operator}({arguments}')
 
 
+class OrderBy(Query):
+    """A class for representing a "ORDER BY" clause."""
+
+    def order_by(self, sorting_properties: Union[str, List[str]], ascending: bool = True):
+        """Concatenate an order by statement.
+
+        :param sorting_properties: A string or a list of strings representing the properties based on which to sort.
+        :type sorting_properties: Union[str, List[str]]
+        :param ascending: Use ascending sorting (if false, uses descending)., defaults to True
+        :type ascending: bool
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: OrderByAvailable
+        """
+        if type(sorting_properties) != list:
+            sorting_properties = [sorting_properties]
+
+        ret = f" ORDER BY {', '.join(sorting_properties)}"
+        ret += " ASC" if ascending else " DESC"
+        return OrderByAvailable(self.query + ret)
+
+
 class Relation(Query):
     """A class for representing a "RELATION" clause."""
 
-    def related(self, label: str = None, ref_name: str = None, properties: Properties = None):
+    def related(self, label: str = None, ref_name: str = None, properties: dict = None):
         """Concatenate an undirectional (i.e. --) graph Relationship, which may be filtered.
 
         :param label: The relationship label (type) in the DB, defaults to None
         :type label: str
         :param ref_name: A reference name to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param properties: A Properties object that represents the set of properties by which the relationship is
-            filtered, defaults to None
-        :type properties: Properties
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            None
+        :type properties: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: RelationAvailable
         """
         return RelationAvailable(self.query + self._directed_relation('none', label, ref_name, properties))
 
-    def related_to(self, label: str = None, ref_name: str = None, properties: Properties = {}):
+    def related_to(self, label: str = None, ref_name: str = None, properties: dict = {}):
         """Concatenate a forward (i.e. -->) graph Relationship, which may be filtered.
 
         :param label: The relationship label (type) in the DB, defaults to None
         :type label: str
         :param ref_name: A reference name to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param properties: A Properties object that represents the set of properties by which the relationship is
-            filtered, defaults to {}
-        :type properties: Properties
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            {}
+        :type properties: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: RelationAvailable
         """
         return RelationAvailable(self.query + self._directed_relation('forward', label, ref_name, properties))
 
-    def related_from(self, label: str = None, ref_name: str = None, properties: Properties = {}):
+    def related_from(self, label: str = None, ref_name: str = None, properties: dict = {}):
         """Concatenate a backward (i.e. <--) graph Relationship, which may be filtered.
 
         :param label: The relationship label (type) in the DB, defaults to None
         :type label: str
         :param ref_name: A reference name to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param properties: A Properties object that represents the set of properties by which the relationship is
-            filtered, defaults to {}
-        :type properties: Properties
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            {}
+        :type properties: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: RelationAvailable
         """
         return RelationAvailable(self.query + self._directed_relation('backward', label, ref_name, properties))
 
-    def related_variable_len(self, hops_len: str):
-        """Concatenate a uni-directional graph Relationship, with a variable length.
+    def related_variable_len(self, min_hops: int = -1, max_hops: int = -1):
+        """Concatenate a uni-directional graph Relationship, with a variable path length.
 
-        :param hops_len: The string representing the desired relationship length
-        :type hops_len: str
+        :param min_hops: The minimal desired number of hops (set -1 for maximum boundary only), defaults to -1
+        :type min_hops: int
+        :param max_hops: The maximal desired number of hops (set -1 for minimal boundary only), defaults to -1
+        :type max_hops: int
 
         :return: A Query object with a query that contains the new clause.
         :rtype: RelationAvailable
         """
-        relation_length = '' if hops_len is None else f'{hops_len}'
+        min_hops_str = '' if min_hops == -1 else str(min_hops)
+        max_hops_str = '' if max_hops == -1 else str(max_hops)
+
+        relation_length = '*' if min_hops == -1 and max_hops == - \
+            1 else (f'*{min_hops_str}'if min_hops == max_hops else f'*{min_hops_str}..{max_hops_str}')
 
         if relation_length:
             realtion_str = f'[{relation_length}]'
@@ -380,7 +408,7 @@ class Relation(Query):
 
         return RelationAvailable(self.query + f'-{realtion_str}-')
 
-    def _directed_relation(self, direction: str, label: str, ref_name: str = None, properties: Properties = {}):
+    def _directed_relation(self, direction: str, label: str, ref_name: str = None, properties: dict = {}):
         """Concatenate a graph Relationship (private method).
 
         :param direction: The relationship direction, can one of 'forward', 'backward' - otherwise unidirectional
@@ -389,9 +417,9 @@ class Relation(Query):
         :type label: str
         :param ref_name: A reference name to be used later in the rest of the query, defaults to None
         :type ref_name: str
-        :param properties: A Properties object that represents the set of properties by which the relationship is
-            filtered, defaults to {}
-        :type properties: Properties
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            {}
+        :type properties: dict
 
         :return: A Query object with a query that contains the new clause.
         :rtype: RelationAvailable
@@ -413,28 +441,151 @@ class Relation(Query):
         return f'-{realtion_str}-'
 
 
+class RelationAfterMerge(Query):
+    """A class for representing a "RELATION AFTER MERGE" clause."""
+
+    def related(self, label: str = None, ref_name: str = None, properties: dict = None):
+        """Concatenate an undirectional (i.e. --) graph Relationship, which may be filtered.
+
+        :param label: The relationship label (type) in the DB, defaults to None
+        :type label: str
+        :param ref_name: A reference name to be used later in the rest of the query, defaults to None
+        :type ref_name: str
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            None
+        :type properties: dict
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: RelationAfterMergeAvailable
+        """
+        return RelationAvailable(self.query + self._directed_relation('none', label, ref_name, properties))
+
+    def related_to(self, label: str = None, ref_name: str = None, properties: dict = {}):
+        """Concatenate a forward (i.e. -->) graph Relationship, which may be filtered.
+
+        :param label: The relationship label (type) in the DB, defaults to None
+        :type label: str
+        :param ref_name: A reference name to be used later in the rest of the query, defaults to None
+        :type ref_name: str
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            {}
+        :type properties: dict
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: RelationAfterMergeAvailable
+        """
+        return RelationAvailable(self.query + self._directed_relation('forward', label, ref_name, properties))
+
+    def related_from(self, label: str = None, ref_name: str = None, properties: dict = {}):
+        """Concatenate a backward (i.e. <--) graph Relationship, which may be filtered.
+
+        :param label: The relationship label (type) in the DB, defaults to None
+        :type label: str
+        :param ref_name: A reference name to be used later in the rest of the query, defaults to None
+        :type ref_name: str
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            {}
+        :type properties: dict
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: RelationAfterMergeAvailable
+        """
+        return RelationAvailable(self.query + self._directed_relation('backward', label, ref_name, properties))
+
+    def related_variable_len(self, min_hops: int = -1, max_hops: int = -1):
+        """Concatenate a uni-directional graph Relationship, with a variable path length.
+
+        :param min_hops: The minimal desired number of hops (set -1 for maximum boundary only), defaults to -1
+        :type min_hops: int
+        :param max_hops: The maximal desired number of hops (set -1 for minimal boundary only), defaults to -1
+        :type max_hops: int
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: RelationAfterMergeAvailable
+        """
+        min_hops_str = '' if min_hops == -1 else str(min_hops)
+        max_hops_str = '' if max_hops == -1 else str(max_hops)
+
+        relation_length = '*' if min_hops == -1 and max_hops == - \
+            1 else (f'*{min_hops_str}'if min_hops == max_hops else f'*{min_hops_str}..{max_hops_str}')
+
+        if relation_length:
+            realtion_str = f'[{relation_length}]'
+        else:
+            realtion_str = ''
+
+        return RelationAvailable(self.query + f'-{realtion_str}-')
+
+    def _directed_relation(self, direction: str, label: str, ref_name: str = None, properties: dict = {}):
+        """Concatenate a graph Relationship (private method).
+
+        :param direction: The relationship direction, can one of 'forward', 'backward' - otherwise unidirectional
+        :type direction: str
+        :param label: The relationship label (type) in the DB
+        :type label: str
+        :param ref_name: A reference name to be used later in the rest of the query, defaults to None
+        :type ref_name: str
+        :param properties: A dict representing the set of properties by which the relationship is filtered, defaults to
+            {}
+        :type properties: dict
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: RelationAfterMergeAvailable
+        """
+        relation_type = '' if label is None else f': {label}'
+        relation_ref_name = '' if ref_name is None else f'{ref_name}'
+        relation_properties = f' {{{str(Properties(properties))}}}' if properties else ''
+
+        if relation_ref_name or relation_type:
+            realtion_str = f'[{relation_ref_name}{relation_type}{relation_properties}]'
+        else:
+            realtion_str = ''
+
+        if direction == 'forward':
+            return f'-{realtion_str}->'
+        if direction == 'backward':
+            return f'<-{realtion_str}-'
+
+        return f'-{realtion_str}-'
+
+
+class Remove(Query):
+    """A class for representing a "REMOVE" clause."""
+
+    def remove(self, properties: Union[str, List[str]]):
+        """Concatenate a remove by statement.
+
+        :param properties: A string or a list of strings representing the properties to remove.
+        :type properties: Union[str, List[str]]
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: RemoveAvailable
+        """
+        if type(properties) != list:
+            properties = [properties]
+        ret = f" REMOVE {', '.join(properties)}"
+        return RemoveAvailable(self.query + ret)
+
+
 class Return(Query):
     """A class for representing a "RETURN" clause."""
 
-    def return_single(self, ref_name: str, returned_name: str = None):
-        """Concatenate a return statement for a single object.
+    def return_literal(self, literal: str):
+        """Concatenate a literal RETURN statement.
 
-        :param ref_name: A reference name to be used to identify the name of the object as was defined earlier in the
-            query
-        :type ref_name: str
-        :param returned_name: The returned name of the object. If none provided, the returned name is {ref_name},
-            defaults to None
-        :type returned_name: str
+        :param literal: A Cypher string describing the objects to be returned, referencing name/names which were
+            defined earlier in the query
+        :type literal: str
 
         :return: A Query object with a query that contains the new clause.
         :rtype: ReturnAvailable
         """
-        ret = ' RETURN ' + (f'{ref_name} as {returned_name}' if returned_name else ref_name)
+        ret = f' RETURN {literal}'
 
-        return Query(self.query + ret)
+        return ReturnAvailable(self.query + ret)
 
-    def return_multiple(self, mappings: List[Mapping]):
-        """Concatenate a return statement for mutiple objects.
+    def return_mapping(self, mappings: List[Mapping]):
+        """Concatenate a RETURN statement for mutiple objects.
 
         :param mappings: The mapping (or a list of mappings) of db property names to code names, to be returned
         :type mappings: List[Mapping]
@@ -442,31 +593,76 @@ class Return(Query):
         :return: A Query object with a query that contains the new clause.
         :rtype: ReturnAvailable
         """
-        if isinstance(mappings, Mapping):
-            return self.return_single(mappings.ref_name, mappings.returned_name)
+        if not isinstance(mappings, list):
+            mappings = [mappings]
 
         ret = ' RETURN ' + \
             ', '.join(
-                f'{mapping.ref_name} as {mapping.returned_name}' if mapping.returned_name else mapping.ref_name
+                f'{mapping[0]} as {mapping[1]}' if mapping[1] else mapping[0].replace(".", "_")
                 for mapping in mappings)
 
-        return Query(self.query + ret)
+        return ReturnAvailable(self.query + ret)
 
 
 class Set(Query):
     """A class for representing a "SET" clause."""
 
-    def set(self, properties: Properties):
+    def set(self, properties: dict, escape_values: bool = True):
         """Concatenate a SET clause, using the given properties map.
 
-        :param properties: The properties dictionary that will be used to set the properties with their corresponding
-            values
-        :type properties: Properties
+        :param properties: A dict to be used to set the variables with their corresponding values
+        :type properties: dict
+        :param escape_values: Determines whether the properties values should be escaped or not, defaults to True
+        :type escape_values: bool
 
         :return: A Query object with a query that contains the new clause.
         :rtype: SetAvailable
         """
-        return SetAvailable(self.query + ' SET ' + Properties(properties).to_str("=", ", "))
+        query = self.query + ' SET ' + Properties(properties).to_str("=", ", ", escape_values)
+
+        if isinstance(self, NodeAfterMergeAvailable) or isinstance(self, OnCreateAvailable) or isinstance(self, OnMatchAvailable) or isinstance(self, SetAfterMergeAvailable):
+            return SetAfterMergeAvailable(query)
+
+        return SetAvailable(query)
+
+
+class SetAfterMerge(Query):
+    """A class for representing a "SET AFTER MERGE" clause."""
+
+    def set(self, properties: dict, escape_values: bool = True):
+        """Concatenate a SET clause, using the given properties map.
+
+        :param properties: A dict to be used to set the variables with their corresponding values
+        :type properties: dict
+        :param escape_values: Determines whether the properties values should be escaped or not, defaults to True
+        :type escape_values: bool
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: SetAfterMergeAvailable
+        """
+        query = self.query + ' SET ' + Properties(properties).to_str("=", ", ", escape_values)
+
+        if isinstance(self, NodeAfterMergeAvailable) or isinstance(self, OnCreateAvailable) or isinstance(self, OnMatchAvailable) or isinstance(self, SetAfterMergeAvailable):
+            return SetAfterMergeAvailable(query)
+
+        return SetAvailable(query)
+
+
+class Skip(Query):
+    """A class for representing a "SKIP" clause."""
+
+    def skip(self, skip_count: Union[int, str]):
+        """Concatenate a skip statement.
+
+        :param skip_count: A non-negative integer or a string of cypher expression that evaluates to a non-negative
+            integer (as long as it is not referring to any external variables)
+        :type skip_count: Union[int, str]
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: SkipAvailable
+        """
+        ret = f" SKIP {skip_count}"
+        return SkipAvailable(self.query + ret)
 
 
 class Unwind(Query):
@@ -484,6 +680,55 @@ class Unwind(Query):
         return UnwindAvailable(self.query + f' UNWIND {variables}')
 
 
+class Where(Query):
+    """A class for representing a "WHERE" clause."""
+
+    def where(self, name: str, comparison_operator: str, value: str):
+        """Concatenate a WHERE clause to the query, created as {name} {comparison_operator} {value}. E.g. x = 'abc'.
+
+        :param name: The name of the object which is to be used in the comparison
+        :type name: str
+        :param comparison_operator: A string operator, according to which the comparison between compared object and
+            the {value} is done, e.g. for "=", we get: {name} = {value}
+        :type comparison_operator: str
+        :param value: The value which is compared against
+        :type value: str
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: WhereAvailable
+        """
+        return self.where_multiple({name: value}, comparison_operator)
+
+    def where_multiple(self, filters: dict, comparison_operator: str = "=", boolean_operator: str = ' AND '):
+        """Concatenate a WHERE clause to the query, created from a list of given property filters.
+
+        :param filters: A dict representing the set of properties to be filtered
+        :type filters: dict
+        :param comparison_operator: A string operator, according to which the comparison between property values is
+            done, e.g. for "=", we get: property.name = property.value, defaults to "="
+        :type comparison_operator: str
+        :param boolean_operator: The boolean operator to apply between predicates, defaults to ' AND '
+        :type boolean_operator: str
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: WhereAvailable
+        """
+        filt = ' WHERE ' + Properties(filters).to_str(comparison_operator, boolean_operator)
+        return WhereAvailable(self.query + filt)
+
+    def where_literal(self, statement: str):
+        """Concatenate a literal WHERE clause to the query.
+
+        :param statement: A literal string of the required filter
+        :type statement: str
+
+        :return: A Query object with a query that contains the new clause.
+        :rtype: WhereAvailable
+        """
+        filt = ' WHERE ' + statement
+        return WhereAvailable(self.query + filt)
+
+
 class With(Query):
     """A class for representing a "WITH" clause."""
 
@@ -513,14 +758,15 @@ class Yield(Query):
         """
         if not isinstance(mappings, list):
             mappings = [mappings]
+
         query = ' YIELD ' + \
-            ', '.join(f'{mapping.ref_name} as '
-                      f'{mapping.returned_name if mapping.returned_name else mapping.ref_name.replace(".", "_")}'
+            ', '.join(f'{mapping[0]} as '
+                      f'{mapping[1] if mapping[1] else mapping[0].replace(".", "_")}'
                       for mapping in mappings)
         return YieldAvailable(self.query + query)
 
 
-class QueryStartAvailable(Match, Merge, Call):
+class QueryStartAvailable(Match, Merge, Call, Create):
     """A class decorator declares a QueryStart is available in the current query."""
 
 
@@ -528,16 +774,20 @@ class CallAvailable(Node, Return, OperatorStart):
     """A class decorator declares a Call is available in the current query."""
 
 
-class CaseWhenAvailable(QueryStartAvailable, With, Unwind, Where, CaseWhen, Return):
+class CaseWhenAvailable(QueryStartAvailable, With, Unwind, Where, CaseWhen, Return, Set):
     """A class decorator declares a CaseWhen is available in the current query."""
 
 
-class DeleteAvailable(Query):
+class CreateAvailable(Node):
+    """A class decorator declares a Create is available in the current query."""
+
+
+class DeleteAvailable(Return, CaseWhen):
     """A class decorator declares a Delete is available in the current query."""
 
 
-class WhereAvailable(Return, Delete, With, Where, OperatorStart, QueryStartAvailable):
-    """A class decorator declares a Where is available in the current query."""
+class LimitAvailable(QueryStartAvailable, With, Unwind, Where, CaseWhen, Return, Set, Skip):
+    """A class decorator declares a Limit is available in the current query."""
 
 
 class MatchAvailable(Node, Return, OperatorStart):
@@ -548,19 +798,19 @@ class MergeAvailable(NodeAfterMerge, Return, OperatorStart):
     """A class decorator declares a Merge is available in the current query."""
 
 
-class NodeAvailable(Relation, Return, Delete, With, Where, OperatorStart, Set, QueryStartAvailable):
+class NodeAvailable(Relation, Return, Delete, With, Where, OperatorStart, OperatorEnd, Set, QueryStartAvailable, Merge, Remove):
     """A class decorator declares a Node is available in the current query."""
 
 
-class NodeAfterMergeAvailable(Relation, Return, Delete, With, Where, OperatorStart, Set, OnCreate, OnMatch, QueryStartAvailable):
+class NodeAfterMergeAvailable(RelationAfterMerge, Return, Delete, With, OperatorStart, OperatorEnd, SetAfterMerge, OnCreate, OnMatch, QueryStartAvailable):
     """A class decorator declares a NodeAfterMerge is available in the current query."""
 
 
-class OnCreateAvailable(Set, OperatorStart):
+class OnCreateAvailable(SetAfterMerge, OperatorStart):
     """A class decorator declares a OnCreate is available in the current query."""
 
 
-class OnMatchAvailable(Set, OperatorStart):
+class OnMatchAvailable(SetAfterMerge, OperatorStart):
     """A class decorator declares a OnMatch is available in the current query."""
 
 
@@ -572,23 +822,47 @@ class OperatorStartAvailable(QueryStartAvailable, Node, With, OperatorEnd):
     """A class decorator declares a OperatorStart is available in the current query."""
 
 
+class OrderByAvailable(Limit, Skip):
+    """A class decorator declares a OrderBy is available in the current query."""
+
+
 class RelationAvailable(Node):
     """A class decorator declares a Relation is available in the current query."""
 
 
-class ReturnAvailable(QueryStartAvailable, With, Unwind, Return):
+class RelationAfterMergeAvailable(NodeAfterMerge):
+    """A class decorator declares a RelationAfterMerge is available in the current query."""
+
+
+class RemoveAvailable(Set, Return):
+    """A class decorator declares a Remove is available in the current query."""
+
+
+class ReturnAvailable(QueryStartAvailable, With, Unwind, Return, Limit, Skip, OrderBy):
     """A class decorator declares a Return is available in the current query."""
 
 
-class SetAvailable(QueryStartAvailable, With, Unwind, Return):
+class SetAvailable(QueryStartAvailable, With, Set, Remove, Unwind, Return):
     """A class decorator declares a Set is available in the current query."""
 
 
-class UnwindAvailable(QueryStartAvailable, With, Unwind, Return):
+class SetAfterMergeAvailable(QueryStartAvailable, OnCreate, OnMatch, With, SetAfterMerge, Unwind, Return):
+    """A class decorator declares a SetAfterMerge is available in the current query."""
+
+
+class SkipAvailable(QueryStartAvailable, With, Unwind, Where, CaseWhen, Return, Set, Remove, Limit):
+    """A class decorator declares a Skip is available in the current query."""
+
+
+class UnwindAvailable(QueryStartAvailable, With, Unwind, Return, Create, Remove):
     """A class decorator declares a Unwind is available in the current query."""
 
 
-class WithAvailable(QueryStartAvailable, With, Unwind, Where, CaseWhen, Return):
+class WhereAvailable(Return, Delete, With, Where, Set, Remove, OperatorStart, QueryStartAvailable):
+    """A class decorator declares a Where is available in the current query."""
+
+
+class WithAvailable(QueryStartAvailable, With, Unwind, Where, Set, Remove, CaseWhen, Return, Limit, Skip, OrderBy):
     """A class decorator declares a With is available in the current query."""
 
 
@@ -596,6 +870,10 @@ class YieldAvailable(QueryStartAvailable, Node, With):
     """A class decorator declares a Yield is available in the current query."""
 
 
+class AnyAvailable(Call, CaseWhen, Create, Delete, Limit, Match, Merge, Node, NodeAfterMerge, OnCreate, OnMatch, OperatorEnd, OperatorStart, OrderBy, QueryStart, Relation, RelationAfterMerge, Remove, Return, Set, SetAfterMerge, Skip, Unwind, Where, With, Yield):
+    """A class decorator declares anything is available in the current query."""
+
+
 class QueryBuilder(QueryStartAvailable):
     """The Query Builder's initial interface."""
 
